-- =============================================================================
--	Ctrl_ISO_OUT.vhd
--	Control pipe hander for ISO-OUT support
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- =============================================================================
library ieee;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity C_ISO_OUT is port (
	reset, clk	: in	std_logic;
	busReset	: in	std_logic; -- Bus reset (0: reset)
--	ReqDIR		: in	std_logic; -- Request direction (0: OUT / 1: IN)
	ReqDEV		: in	std_logic; -- Request for DEVICE (1: Valid)
	ReqIF		: in	std_logic; -- Request for I/F (1: Valid)
	ReqEP		: in	std_logic; -- Request for E/P (1: Valid)
	ReqType		: in	std_logic; -- Request type (0: std. / 1: class)
	bRequest	: in	std_logic_vector( 7 downto 0); -- bRequest
	wValue		: in	std_logic_vector(15 downto 0);
	wIndex		: in	std_logic_vector(15 downto 0);
	wLength		: in	std_logic_vector(15 downto 0);
	ByteData	: in	std_logic_vector( 7 downto 0);
	DataValid	: in	std_logic;	-- ByteData was valid (1: Valid)
	EOP			: in	std_logic;	-- End Of Packet
--	ERR16		: in	std_logic;	-- CRC
	Config		: in	std_logic;	-- Configuration status (0 Unconfigured, 1: Configured)

	ReqCanHnd	: out	std_logic;	-- Request can be handled
	ReqReject	: out	std_logic;	-- Request is rejected
	Freq4896	: out	std_logic;	-- 0: 48kHz / 1: 96kHz
	ALT1		: out	std_logic_vector(1 downto 0);
	TxAddr		: in	std_logic_vector(7 downto 0);
	data8		: out	std_logic_vector(7 downto 0);
	MaxLen		: out	std_logic_vector(7 downto 0)
);end entity;

architecture RTL of C_ISO_OUT is

component conf_desc port(
		address	: in  std_logic_vector(6 downto 0);
		data8	: out std_logic_vector(7 downto 0)
);end component;

-- Standard request --
constant GET_STATUS			: std_logic_vector(7 downto 0) := X"00";
constant CLEAR_FEATURE		: std_logic_vector(7 downto 0) := X"01";
constant SET_FEATURE		: std_logic_vector(7 downto 0) := X"03";
constant GET_DESCRIPTOR		: std_logic_vector(7 downto 0) := X"06";
constant GET_INTERFACE		: std_logic_vector(7 downto 0) := X"0A";
constant SET_INTERFACE		: std_logic_vector(7 downto 0) := X"0B";
-- Audio class specific request --
constant SET_CUR			: std_logic_vector(7 downto 0) := X"01";
constant GET_CUR			: std_logic_vector(7 downto 0) := X"81";
constant SET_MIN			: std_logic_vector(7 downto 0) := X"02";
constant GET_MIN			: std_logic_vector(7 downto 0) := X"82";
constant SET_MAX			: std_logic_vector(7 downto 0) := X"03";
constant GET_MAX			: std_logic_vector(7 downto 0) := X"83";
constant SET_RES			: std_logic_vector(7 downto 0) := X"04";
constant GET_RES			: std_logic_vector(7 downto 0) := X"84";
-- Length of Configuration descriptor --
-- Length should be met to actual descriptor length
constant CONFIG_LENGTH	: std_logic_vector(7 downto 0) := X"70";

type typeEP0 is (IDLE,
				SET_FREQ0, SET_FREQ1, SET_FREQ2, SET_FREQ3, SET_FREQ4,
				GET_FREQ0,
				CLR_FEATURE0, SET_FEATURE0,
				GET_STATUS0
);

signal	fsmEP		: typeEP0;
signal	iReqReject	: std_logic;
signal	iFreq4896, cFreq4896	: std_logic;	-- 0: 48kHz / 1: 96kHz
signal	iALT1		: std_logic_vector(1 downto 0); -- Holds alternate setting of IF#01
signal	CDData8		: std_logic_vector(7 downto 0);

begin

process(reset, clk)
begin
if(reset = '0')then
	ReqCanHnd <= '0'; ReqReject	<= '0'; iReqReject	<= '0';
	iALT1		<= "00";
	cFreq4896	<= '0'; iFreq4896 <= '0';
	data8		<= X"00";
	MaxLen		<= X"00";
	fsmEP <= IDLE;
elsif(clk'event AND clk = '1')then
	case fsmEP is
		when IDLE =>
			ReqCanHnd	<= '0'; ReqReject	<= '0'; iReqReject	<= '0';
			data8	<= X"00";
			MaxLen	<= X"00";
			if(ReqEP = '1')then
				if(ReqType = '1')then -- class specific
---------------------------- Audio Class specific request -----------------------------------------------
					if(wIndex(3 downto 0) = X"1")then --> Endpoint#01(ISO-OUT)
						ReqCanHnd <= '1';--ReqCanHnd,ReqReject,MaxLen should be asserted at next clock
						case bRequest is
							when SET_CUR	=>	-- SET_CUR
								if( (wValue = X"0100")	--> CS=0x01: SAMPLING_FREQ_CONTROL/Master ch.
								AND (wLength= X"0003") )then
									fsmEP <= SET_FREQ0;
								else
									ReqReject <= '1';
								end if;

							when GET_CUR	=>	-- GET_CUR
								if( (wValue = X"0100")	--> CS=0x01: SAMPLING_FREQ_CONTROL/Master ch.
								AND (wLength= X"0003") )then
									MaxLen <= X"03";
									fsmEP <= GET_FREQ0;
								else
									ReqReject <= '1';
								end if;
							when others	=>
								ReqReject <= '1';
						end case;
					end if;
---------------------------------------------------------------------------------------------------------
				end if;--/cls
				if(Config = '0')then	-- any request should be failed when unconfigured
					ReqReject <= '1'; fsmEP <= IDLE;
				end if;
			elsif(ReqIF = '1')then
				if(ReqType = '0')then -- std.
					case bRequest is
--- SET_INTERFACE ---
						when SET_INTERFACE =>
							if(wIndex(7 downto 0) = X"01")then --> I/F#01(ISO-OUT)
								ReqCanHnd<='1';
								iALT1 <= wValue(1 downto 0);
							end if;
--- GET_INTERFACE ---
						when GET_INTERFACE =>
							if(wIndex(7 downto 0) = X"01")then --> I/F#01(ISO-OUT)
								ReqCanHnd<='1';
								MaxLen <= X"01";
								data8 <= "000000"&iALT1;
							end if;

						when others	=> null;
					end case;
				end if;--/std.
				if(Config = '0')then	-- any request should be failed when unconfigured
					ReqReject <= '1';
				end if;
			elsif(ReqDEV = '1')then
				ReqReject <= '1';
				if(ReqType = '0')then -- std.
					case bRequest is
--- GET_DESCRIPTOR ---
						when GET_DESCRIPTOR =>
							if(wValue = X"0200")then -- CONFIGURATION
								ReqCanHnd <= '1'; ReqReject <= '0';
								if(wLength > X"00"&CONFIG_LENGTH)then
									MaxLen <= CONFIG_LENGTH;
								else
									MaxLen <= wLength(7 downto 0);
								end if;
								data8 <= CDData8;
							end if;
						when others => null;
					end case;
				end if;--/std.
			end if;

---------------------------------------------------------------------------------------------------------
--- SET_FREQ ---
		when SET_FREQ0 => -- Check 1st byte
			if(DataValid = '1')then
				if(ByteData = X"80")then --48kHz
					iFreq4896 <= '0';	iReqReject <= '0';
					fsmEP <= SET_FREQ1;
				elsif(ByteData = X"00")then --96kHz
					iFreq4896 <= '1';	iReqReject <= '0';
					fsmEP <= SET_FREQ1;
				else
					iReqReject <= '1';
					fsmEP <= SET_FREQ3;
				end if;
			end if;
			if(ReqEP = '0')then
				fsmEP <= SET_FREQ4;--Complete
			end if;

		when SET_FREQ1 => -- Check 2nd byte
			if(DataValid = '1')then
				if( ((ByteData = X"BB") AND (iFreq4896 = '0'))
				 OR ((ByteData = X"77") AND (iFreq4896 = '1')) )then
					fsmEP <= SET_FREQ2;
				else
					iReqReject <= '1';
					fsmEP <= SET_FREQ3;
				end if;
			end if;
		when SET_FREQ2 => -- Check last byte
			if(DataValid = '1')then
				if( ((ByteData = X"00") AND (iFreq4896 = '0'))
				OR  ((ByteData = X"01") AND (iFreq4896 = '1')) )then
					iReqReject <= '0';
					fsmEP <= SET_FREQ3;
				else
					iReqReject <= '1';
					fsmEP <= SET_FREQ3;
				end if;
			end if;

		when SET_FREQ3 =>
			ReqReject <= iReqReject;
			if(EOP = '1')then	-- Wait End Of Packet of DATA1
				fsmEP <= SET_FREQ0;-- Return to first state to prepare retry
			end if;
--- Set current freq when transaction has ended ---
		when SET_FREQ4 =>
			if(iReqReject = '0')then
				cFreq4896 <= iFreq4896; -- current freq.
			end if;
			fsmEP <= IDLE;

--- GET_FREQ ---
		when GET_FREQ0 =>
			case TxAddr(1 downto 0) is
			  when "00" =>
				if(cFreq4896 = '0')then
					data8 <= X"80";
				else
					data8 <= X"00";
				end if;
			  when "01" =>
				if(cFreq4896 = '0')then
					data8 <= X"BB";
				else
					data8 <= X"77";
				end if;
			  when "10" =>
				if(cFreq4896 = '0')then
					data8 <= X"00";
				else
					data8 <= X"01";
				end if;
			  when others => data8 <= X"00";
			end case;
			if(ReqEP = '0')then
				fsmEP <= IDLE;
			end if;
--
		when others	 => fsmEP <= IDLE;
	end case;
--------------------------
	if(busReset = '0')then
		cFreq4896<= '0'; iFreq4896 <= '0';
		iALT1	<= "00";
		fsmEP	<= IDLE;
	end if;
end if;
end process;
Freq4896 <= cFreq4896;
ALT1 <= iALT1;

CD:conf_desc port map(TxAddr(6 downto 0), CDData8);

end RTL;
