-- ============================================================================
--	I2C.vhd   CODEC initialize routine (AIC3204)
--	Dedicated to FPGA mag. (CQ pub.)
--
--	2017-10-25	V1.0	Y.Okamura	SONIC ONE (http://sonic1.biz)
-- ============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
library work;

entity I2C is port(
	reset, clk	: in	std_logic;
	init		: in	std_logic;-- 1: Start init.
	status		: out	std_logic;
	SCL			: out	std_logic;
	SDA			: inout	std_logic
);end entity;

architecture RTL of I2C is

component I2C_out port (
	reset	: in	std_logic; -- reset (active low)
	clk		: in	std_logic; -- Expected freq.=48MHz
	data8	: in	std_logic_vector(7 downto 0); -- 8bits data
	start	: in	std_logic; -- Start I2C (1: start)
	stop	: in	std_logic; -- Stop I2C (1: stop)
	busy	: out	std_logic; -- Busy (1: busy)
	err		: out	std_logic; -- Error (1: Error)
	SCL		: out	std_logic;
	SDA		: inout	std_logic
);end component;

type state_type is (START0, DADDR0, DADDR1, DADDR2, DATA0, DATA1, DATA2, STOP0, STOP1, IDLE);
signal	fsmCMD	: state_type;
signal	start, stop, busy	: std_logic;
signal	err		: std_logic;
signal	cmd		: std_logic_vector(7 downto 0);
signal	cmd_data	: std_logic_vector(7 downto 0);
signal	iData, iAddr	: std_logic_vector(7 downto 0);

begin
MainSQ:process(reset, clk)
begin
	if(reset = '0')then
		fsmCMD   <= START0;
		start <= '0'; stop <= '0';
		cmd <= X"00"; iAddr <= X"00";
		status <= '0';
	elsif(clk'event AND clk = '1')then
		case fsmCMD is
---------------------------------------------------------
			when START0 =>
				start <= '0'; stop <= '0';
				iAddr <= X"00";
				if(init = '1')then
					fsmCMD   <= DADDR0;
				end if;
--------- Device address --------------------------------
			when DADDR0 =>
				if(busy = '0')then
					start <= '1';
					cmd <= X"30";-- Device address & write command
					fsmCMD <= DADDR1;
				end if;
			when DADDR1 =>
				if(busy = '1')then
					start <= '0';
					fsmCMD <= DADDR2;
				end if;
			when DADDR2 =>
				if(busy = '0')then
					if(err = '1')then
						fsmCMD <= IDLE;
					else
						fsmCMD <= DATA0;
					end if;
				end if;
---------------------------------------------------------
			when DATA0 =>
				if(busy = '0')then
					cmd <= iData;
					fsmCMD <= DATA1;
				end if;
			when DATA1 =>
				if(busy = '1')then
					iAddr <= iAddr + X"01";
					fsmCMD <= DATA2;
				end if;
			when DATA2 =>
				if(busy = '0')then
					if(iAddr(0) = '0')then
						stop <= '1';
						fsmCMD <= STOP0;
					else
						fsmCMD <= DATA0;
					end if;
				end if;
----------------------------------------------------------
			when STOP0 =>
				if(busy = '1')then
					stop <= '0';
					fsmCMD <= STOP1;
				end if;
			when STOP1 =>
				if(busy = '0')then
					if(iAddr = X"32")then
						status <= '1';
						fsmCMD <= IDLE;
					else
						fsmCMD <= DADDR0;
					end if;
				end if;
--------------------------------------------------------
			when IDLE =>
				start <= '0'; stop <= '0';
				status <= '0';
--				if(init = '0')then		-- When device is unconfigured, enter power down sequence.
--					fsmCMD <= PDOWN0;
--				end if;
--				if(sspnd= '0')then		-- When device is suspended, enter power down sequence.
--					fsmCMD <= PDOWN0;
--				end if;
			when others =>	fsmCMD <= IDLE;
		end case;
	end if;
end process;

CO:I2C_out port map(reset, clk, cmd, start, stop, busy, err, SCL, SDA);
---- I2C commands ----
process(iAddr)
begin
	iData <= "--------";
	case iAddr is
		when X"00" => iData <= X"00";-- Initialize to Page 0
		when X"01" => iData <= X"00";
		when X"02" => iData <= X"01";-- Initialize the device through software reset
		when X"03" => iData <= X"01";
		when X"04" => iData <= X"0B";-- Power up the NDAC divide with value 1
		when X"05" => iData <= X"81";
		when X"06" => iData <= X"0C";-- Power up the MDAC divider with value 4
		when X"07" => iData <= X"84";
		when X"08" => iData <= X"0D";-- Program the OSR of DAC to 64 (#1)
		when X"09" => iData <= X"00";
		when X"0A" => iData <= X"0E";-- Program the OSR of DAC to 64 (#2)
		when X"0B" => iData <= X"40";
		when X"0C" => iData <= X"3C";-- Set the DAC Mode to PRB_P8
		when X"0D" => iData <= X"08";
		when X"0E" => iData <= X"1B";-- Set I2S/24bit
		when X"0F" => iData <= X"20";
		when X"10" => iData <= X"1C";-- Offset
		when X"11" => iData <= X"00";
		when X"12" => iData <= X"00";-- Select Page 1
		when X"13" => iData <= X"01";
		when X"14" => iData <= X"01";-- Disable Internal Crude AVdd in presence of external AVdd supply or before powering up internal AVdd LDO
		when X"15" => iData <= X"08";
		when X"16" => iData <= X"02";-- Enable Master Analog Power Control
		when X"17" => iData <= X"01";
		when X"18" => iData <= X"7B";-- Set the REF charging time to 40ms
		when X"19" => iData <= X"01";
		when X"1A" => iData <= X"14";-- HP soft stepping settings
		when X"1B" => iData <= X"25";
		when X"1C" => iData <= X"0A";-- Set the Input Common Mode to 0.9V and Output Common Mode for Headphone
		when X"1D" => iData <= X"00";
		when X"1E" => iData <= X"0C";-- Route Left DAC to HPL (#1)
		when X"1F" => iData <= X"08";
		when X"20" => iData <= X"0D";-- Route Right DAC to HPL (#2)
		when X"21" => iData <= X"08";
		when X"22" => iData <= X"03";-- Set the DAC PTM mode to PTM_P1 (#1)
		when X"23" => iData <= X"08";
		when X"24" => iData <= X"04";-- Set the DAC PTM mode to PTM_P1 (#2)
		when X"25" => iData <= X"08";
		when X"26" => iData <= X"10";-- Set the HPL gain to 0dB (#1)
		when X"27" => iData <= X"00";
		when X"28" => iData <= X"11";-- Set the HPR gain to 0dB (#2)
		when X"29" => iData <= X"00";
		when X"2A" => iData <= X"09";-- Power up HPL and HPR drivers
		when X"2B" => iData <= X"30";
		when X"2C" => iData <= X"00";-- Select Page 0
		when X"2D" => iData <= X"00";
		when X"2E" => iData <= X"3F";-- Power up the Left and Right DAC Channels with route the Left Audio digital data to Left Channel DAC and Right Audio digital data to Right Channel DAC
		when X"2F" => iData <= X"D6";
		when X"30" => iData <= X"40";-- Unmute the DAC digital volume control
		when X"31" => iData <= X"00";
		when others  => null;
	end case;
end process;

end RTL;
